use serde_json::Value;
use songbird::input::reader::MediaSource;
use std::{
    io::BufReader,
    process::{Command, Stdio},
};
use symphonia_core::io::ReadOnlySource;
use tokio::process::Command as TokioCommand;

pub async fn ytdl(query: &str) -> Result<Value, std::io::Error> {
    let mut cmd = TokioCommand::new("youtube-dl");
    let cmd = cmd
        .arg("-j")
        .arg("-f")
        .arg("bestaudio")
        .arg(query)
        .stdout(Stdio::piped())
        .stderr(Stdio::piped());
    let child = cmd.spawn()?;
    let output = child.wait_with_output().await?;
    let stdout = String::from_utf8(output.stdout).unwrap();
    let stderr = String::from_utf8(output.stderr).unwrap();
    if output.status.success() {
        Ok(serde_json::from_str(&stdout)?)
    } else {
        Err(std::io::Error::new(
            std::io::ErrorKind::Other,
            format!(
                "ytdl command exited with status {}: {}",
                output.status, stderr
            ),
        ))
    }
}

pub async fn ffmpeg_pcm(url: &str) -> Result<Box<dyn MediaSource + Send>, String> {
    let mut cmd = Command::new("ffmpeg");
    let cmd = cmd
        .arg("-reconnect")
        .arg("1")
        .arg("-reconnect_streamed")
        .arg("1")
        .arg("-reconnect_delay_max")
        .arg("5")
        .arg("-i")
        .arg(url)
        .arg("-f")
        .arg("s16le")
        .arg("-ar")
        .arg("48000")
        .arg("-ac")
        .arg("2")
        .arg("-acodec")
        .arg("pcm_f32le")
        .arg("pipe:1")
        .stdout(Stdio::piped())
        .stderr(Stdio::null());
    let child = match cmd.spawn() {
        Ok(child) => child,
        Err(error) => {
            return Err(format!("{}", error));
        }
    };
    let out = match child.stdout {
        Some(out) => out,
        None => return Err("subprocess::ffmpeg_pcm: failed to get child stdout".to_string()),
    };
    let buf = BufReader::with_capacity(16384 * 8, out);
    let buf: Box<dyn MediaSource + Send> = Box::new(ReadOnlySource::new(buf));
    Ok(buf)
}
